
# q-expansion of newform 270.6.a.i, downloaded from the LMFDB on 11 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 270
weight = 6
poly_data = [-11, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 18], 1]]

hecke_ring_character_values = None
aps_data = [[-4, 0], [0, 0], [25, 0], [-10, 1], [60, 9], [-10, 5], [849, -25], [-229, -15], [693, 70], [4530, -75], [-4933, 85], [-4150, 28], [-540, -223], [-4720, -277], [12672, 100], [21657, 125], [-6690, 233], [6821, 230], [29510, -318], [50640, 315], [32300, -179], [-1195, 595], [51555, -1090], [84900, -893], [56240, -996], [69780, 656], [49460, 1492], [63480, 730], [55145, -250], [34242, 1800], [146630, -54], [8670, -2548], [10527, -2225], [91004, -4040], [-325740, 1531], [-249796, -2830], [218960, 3581], [445610, 736], [-323043, -3260], [205551, 3235], [-496020, -1530], [33833, 550], [-300930, -5000], [378650, -655], [-214605, 1525], [459632, 7070], [-233119, -10405], [419630, -9392], [281259, 15290], [223703, 14020], [-235986, -14100], [-508350, -5078], [-920569, 13060], [206280, 24846], [-365277, -10995], [794148, -15920], [-1330290, 11850], [374795, 3925], [-1209460, -13271], [527070, -6425], [729260, -21623], [1169919, 1395], [-730330, 14340], [2180130, -16259], [368240, 48070], [1619073, -13755], [-2766580, -6470], [526310, -20831], [450252, 21640], [-1782259, -17720], [51666, 62890], [102930, -16554], [-4303570, -13419], [3436400, -1710], [-2258899, 24535], [-2025819, 1170], [3754080, -34531], [-1470670, 40237], [-2132550, 37352], [-1104211, -78110], [-442500, -30683], [-2224789, -30080], [2388150, 20177], [1825490, 72515], [573899, 52875], [492267, -71560], [-658830, 58161], [-1521940, 30666], [1445130, 48834], [-190210, 55258], [1097619, -64570], [137700, 104877], [-2254660, -130561], [-4801470, -78038], [-7401565, -24285], [-140355, 117550], [2277900, 46832], [1915500, -83711], [1651190, -16777], [-1266430, -25670], [1157960, 88939], [-2502858, -53370], [-10343184, 12350], [-746910, -159369], [-5667649, -119225], [-5971360, 20849], [-10870587, -91490], [1900359, 53605], [-23550, -106981], [8578337, -69760], [-1449400, -187613], [4960520, -97772], [-3637407, -208515], [3646592, -198850], [-9475315, 90835], [8233800, -68948], [-8717590, 12675], [3498873, 44990], [-3468555, 46135], [3378570, 209241], [-1044022, 258010], [7115510, 60817], [528594, 162540], [11706015, -122040], [10515425, -93585], [14337510, 3040], [19413062, 10590], [-9458610, -215332], [-11053840, -231114], [1940840, -182088], [2770661, -50175], [542808, -264440], [5600885, 67135], [12611930, -283882], [8943750, 318047], [4308209, 184130], [7631493, 68615], [6091940, 380849], [764667, -418245], [-24378540, -104339], [8810336, -91310], [17293470, 175432], [-1932760, -121189], [4979481, -196630], [-6038902, 179560], [-12828480, -289718], [-12812260, -135498], [-18056127, 143525], [-8554675, -339335], [39516837, -20000], [11002520, 270187], [180270, -163925], [-22141930, -78351], [-4302771, 307520], [-1347460, 355216], [-4056210, 209819], [3226664, 67440], [-5357340, -206655], [29082560, -272294], [3149490, -840564], [-34317273, -62110], [7985622, -470640], [8397050, -206792], [48593910, 102504], [-12749982, -43210], [-17901039, 149280], [26165777, 315095], [-29772400, -531333], [1523831, 189140], [-37793517, 301115], [-36980940, -147472], [-14468833, 269790], [-1763490, 278177], [-38438380, 109396], [-957295, -784225], [36864810, 55879], [-17913145, -141215], [9300150, 924403], [-10861540, -537913], [-23874682, -305890], [-1418440, -83603], [-12495240, -119062], [-355870, 82104], [-1611066, 622220], [-25065345, -776720], [33926220, 451112], [-9981250, 1074051], [-45470920, -47354], [-9042523, -318660], [45367050, -283827], [19880510, -173688], [-8887935, 95260], [-1951771, -1396805], [-11717880, -1211688], [33044259, 253630], [-20807625, 67725], [54242537, 46180], [-68270560, -15446], [56610822, -35690], [56566308, 26070], [57900420, 693377], [37097012, 175150], [-77847970, 370180], [-12992605, -563720], [-40050930, 514691], [-15096534, -1349260], [-14873620, 195990], [-35338881, 627710], [61498620, 58000], [-31009348, -407600], [-23778250, -46907], [17411040, 542963], [-24040120, -44940], [-29103912, 593170], [98760780, 231264], [-6223003, 1186330], [-36521380, -52861], [27949200, -457604], [-42360471, 917570], [-39261579, -844255], [-53065582, -1109700], [23451119, 1279575], [110784270, -358593], [22664540, 1231605], [75944631, -442290], [-57880381, 287630], [-19218021, -1448945], [-35676360, -503105], [100859270, 752122], [-57653100, -345476], [2940860, -717845], [-26327293, 1443615], [52561952, -139120], [-30720030, 1102232], [86198870, 589183], [-10857669, -142590], [76421165, 400360], [-93401985, 14875], [49665030, -253792], [75386220, -1023417], [16441440, -1486490], [-98085265, 334515], [73625510, -1099078], [-108018223, 149490], [-53004510, -510670], [-103358130, 969818], [22349120, 1841562], [-42865710, -661197], [66824828, -1058780], [100557087, 409500], [-58597750, 1032330], [-131901930, 484526], [-27118965, 1014600], [57048161, 242550], [-13471047, 1484415], [-159556200, -831628], [-95622151, -935560], [180479210, 380379], [-44340987, -2535355], [-90105190, 111543], [76836140, -1493151], [-62017539, -1186970], [-170930137, 379450], [-84920620, 1236964], [-57728925, 561045], [31271612, 780720], [-31495800, 1428208], [41928690, -466040], [101240990, -417456], [-93122418, 585170], [-53994481, -670940], [-78703570, 1099546], [-93239320, -1983879], [-6199381, -2930355], [-122256820, -849863], [232050290, -240673], [59083833, -2464000], [-90131611, 2463820], [-195525763, -567650], [-60967680, -1440691], [150855108, -1775870], [-218244565, 703515], [9160437, 1772650], [-46853581, -2300520], [-125045020, -300168], [141485130, 1193905], [86304170, 1486922], [121095522, 62540], [-99891619, 1957945], [42358440, 3367768], [202360560, 1463124], [205235673, -963420], [-24079110, -1247000], [61325970, 4085646], [-160874320, 1779315], [30518580, 181470], [12080519, -1601985], [159735726, -2093180], [122635920, -1275784], [11305190, -1558241], [46590440, -4452659], [-7413507, 1721165], [219343448, 421940], [-30333669, 5436610], [40705961, -1925], [123869360, 2831351], [119079360, 3500850], [-55064257, -836020], [-212301180, 1815147], [182552840, -150533], [1172823, -5190], [152358060, -2244782], [14418840, -1822302], [56318510, 2219337], [84963903, -290910], [149939507, 3082790], [-145610040, 2688420], [-68665080, -5017125], [136042070, -783281], [-10121070, -5284748], [-112315831, -2957975], [99665000, 1758182], [-93832860, -139030], [229885040, -934190], [105591981, 2289855], [67714010, -745070], [-173316607, -1061985], [-264707320, 1059450], [-50287440, -733640], [-243851841, 1169205], [-96799630, 5441340], [-14313078, 7102680], [-284489932, 1955230], [52341696, -235420], [-173012560, 1153290], [148882440, -1449690], [139049555, -1253800], [-396514455, -1240985], [332887418, 489380], [-215906560, -2155821], [-398448280, 1025682], [17771349, -2550655], [157599030, -959883], [236687453, -178265], [-1849206, -3153680], [-120665760, 3369067], [-42837649, -524940], [-195291940, 3069770], [6497340, -365954], [-228905037, -1988635], [-146768632, 599530], [291720590, 3310222], [-150488670, 2378097], [-305385520, 722490], [45519110, -1200840], [-341296143, -552005], [-264159690, 2038949], [-128003310, -902157], [216717918, -1207950], [-210959679, -2010560], [-1869130, -570093], [-298376490, 1649542], [164067300, -623880], [-213444930, -560774], [-62418700, 431878], [-186350410, -1877411], [194400633, 446775], [-197587270, -2082404], [377180642, 578660], [151866090, 1313338], [269697767, -430025], [445106079, -755300], [29402160, -4915203], [-92792101, -1402835], [-164889760, 2676402], [24487710, -1121708], [55619310, 6424540], [258948020, 5829863], [405566790, -4449698], [305267030, -87190], [91027140, 2009586], [214859499, 2547075], [287053190, -1146881], [40519653, -4048305], [-164948980, 211330], [220988616, 1999300], [29969213, -5665095], [5706950, 5432629], [-66591640, -11207957], [476722008, -1369760], [500928710, 2430070], [-76679799, -2877785], [293742990, -1171968], [159453230, -4378796], [-237834390, 2014002], [510678920, 180035], [-44840851, 8722075], [569207610, -427375], [377559917, 3287255], [284608740, 1268654], [-201675790, -8961540], [-295539066, -5976470], [-141523390, 8068639], [-14112330, -1428200], [576716850, -565301], [-124958185, 7368285], [245487230, -2342916], [-183328050, -6732670], [-237754180, -7481442], [-722178450, 1437055], [103817390, 9441508], [128885775, 8253075], [145972845, -6506100], [171176423, 8883525], [-101625910, -8340098], [157122150, -4882423], [-119286210, 4266970], [-295819120, 5281752], [-171111045, 6271335], [258283971, 2379880], [420522780, 2939298], [-108901510, 5153522], [-383278437, 2963620], [-197071470, -6757798], [-337893010, 2040642], [221866119, -2126145], [190008285, -1133870], [377272230, 668076], [-87281935, -11534715], [10180800, -11712655]]
